
#include "sssd.h"
#include "internals.h"

/*
    Play sample referenced by id 'repeat_count' times. If the repeat_count is
    0 playing will loop forever. Return (negative) playing ID or 0.
*/

int sssd_playSamplePP( struct sssdaudio* ga, int sample_id, int flags, int repeat_count, int volume, int pan, int playStart, int playEnd, int loopStart, int loopEnd, int priorityQueue ) 
{
    int playing_id=0;
    struct ga_playing* curplaying;
    struct ga_sample* cursample;
    int sampleDataI, byps;

    if( audioPaused( ga ) )
        return(0);

    /* Sanity check */
    if( (sample_id < 0) || (sample_id >= ga->num_samples) )
        return(0);

    /* Clip parameters */
    if (volume<0) {
     volume=0;
    } else if (volume>256) {
     volume=256;
    }
    
    pan+=128;
    if (pan<0) {
     pan=0;
    } else if (pan>256) {
     pan=256;
    }


    LOCK_BUFFERS(ga);

    /* Check if there is a slot free */
    if( ga->num_playing < ga->max_voices )
    {
    	/* Update playing count (count down, wrap around without uniqueness check) */
    	ga->playing_count--;
    	if (ga->playing_count>0) ga->playing_count=-1;
        playing_id=ga->playing_count;
    	
        /* Fill slot with info */
        cursample  = &ga->samples[ sample_id ];
        curplaying = &ga->playing[ ga->num_playing ];

	    sampleDataI=(int)cursample->data;

        curplaying->rootpos      = (void*) sampleDataI;
        curplaying->endpos       = cursample->dataendmarker;        
	    curplaying->repeat_count = repeat_count;
        curplaying->sample_id    = sample_id;
        curplaying->playing_id   = ga->playing_count;

        byps=ga->byps;

        if(cursample->flags & gaCHAN_CONVERT) {
          if(ga->channels==2) byps>>=1;
          else
          if(ga->channels==1) byps<<=1;
          curplaying->voiceflags=PLAYFLAG_CHANCHANGE;
        }
            
        if(playStart!=SAMPLE_DEFAULTPOS) playStart*=byps;
        if(playEnd  !=SAMPLE_DEFAULTPOS) playEnd  *=byps;
        if(loopStart!=SAMPLE_DEFAULTPOS) loopStart*=byps;
        if(loopEnd  !=SAMPLE_DEFAULTPOS) loopEnd  *=byps;


        /* curplaying->curpos    = cursample->data;*/
	    if(playStart!=SAMPLE_DEFAULTPOS) curplaying->playpos=curplaying->curpos=(void *)(sampleDataI+playStart);
	    else curplaying->playpos=curplaying->curpos=(void *)(sampleDataI);

	    if(playEnd!=SAMPLE_DEFAULTPOS) curplaying->playend=(void *)(sampleDataI+playEnd);
	    else curplaying->playend=cursample->dataendmarker;

	    if(loopStart!=SAMPLE_DEFAULTPOS) curplaying->looppos=(void *)(sampleDataI+loopStart);
	    else curplaying->looppos=(void*) (sampleDataI);

	    if(loopEnd!=SAMPLE_DEFAULTPOS) curplaying->loopend=(void *)(sampleDataI+loopEnd);
	    else curplaying->loopend=cursample->dataendmarker;

	    curplaying->priorityQueue=priorityQueue;

        if(ga->channels==2) {
           curplaying->a=(volume*(256-pan))>>8;
           curplaying->b=(volume*(pan))>>8;
        } else {
           curplaying->b=volume;
        }

        ga->num_playing++;        

    } else {
       #ifndef __NOPRINTING__
       fprintf(stderr,"Maximum voices reached: '%d' will not be played\n",sample_id);
       #endif
       ga->lastERR=ERR_VOICES_FULL;
       playing_id=ERR_VOICES_FULL;
    }

    UNLOCK_BUFFERS(ga);
    
    return (playing_id);
}
