
#include "sssd.h"
#include "internals.h"

/*
    Worker-thread that plays the audio
*/

void* playthrd8m( struct sssdaudio* ga )
{
    unsigned char *playbuffer;
    struct ga_playing* curplaying;
    short leftsample;
    unsigned char *curposleft;
    unsigned char *tmpsp;
    int i, j;
    int local_num_playing, abufsize;
    _REGISTER int sampAccumL;
    int fillBackground=1;

    int CBRflag, blockUsec;

#ifndef __NOPRINTING__
puts("playthrd8m starting");
#endif

    playbuffer=ga->playBuffer;
    abufsize=ga->audioBufferSize;
    CBRflag=ga->flags & SSSDSYS_CONSTRATE;
    blockUsec=((SLEEP_TYPE)abufsize*1000000)/ga->freq;
    
#ifndef __NOPRINTING__
    printf("block usec: %d   (size=%d  freq=%d)\n",blockUsec,abufsize,ga->freq);
#endif

    /* Loop ... playing sound */
playloop:

        LOCK_BUFFERS(ga);

        if(ga->threadSignal==1) {
	  ga->threadSignal=2;
	  UNLOCK_BUFFERS(ga);
#ifndef __NOPRINTING__
puts("playthrd8m exiting");
#endif
	  EXIT_THREAD(ga);
	}

        local_num_playing=ga->num_playing;

      if(fillBackground==1) {
        if( ga->background_stream == NULL )
        {
	   memset((void *)playbuffer,0x80,abufsize);
	   fillBackground=0;
        }
        else
        {
            fillBackground=1;
	    /* Preset from background stream */
            if( fread(playbuffer, 1, abufsize, ga->background_stream)
                < abufsize )
            {
                /* Close */
                pclose( ga->background_stream );
                ga->background_stream = NULL;
                ga->background_stream_bytes = 0;
                /* Clear to zero after all */
	        memset((void *)playbuffer,0x80,abufsize);
            } else {
            	/* Store stream bytes */
            	ga->background_stream_bytes += abufsize;
            }
        }
      }

        /* Needs mixing? */
        if( local_num_playing > 0 )
        {
            /* Create playbuffer */
            curposleft  = playbuffer;
	    fillBackground=1;

            for( i = abufsize; i > 0; i-- )
            {
                curplaying = ga->playing;
                sampAccumL=sample8to16(*curposleft)<<8;

              for( j = 0; j < local_num_playing; j++ )
                {
                    /* Get source samples and advance sample positions */
                    tmpsp=(unsigned char *)curplaying->curpos;
                    if(!(curplaying->voiceflags&PLAYFLAG_CHANCHANGE)) {
                       leftsample=sample8to16(*tmpsp);
                    } else {
                       int i=sample8to16(*tmpsp);
                       tmpsp++;
                       i+=sample8to16(*tmpsp);
                       leftsample=i>>1;
                    }
                    tmpsp++;
                    curplaying->curpos=(void *)tmpsp;

                    /* Mangle left and right components with the  */
                    /* pan/volume coefficients                    */
                    sampAccumL += (int)leftsample  * curplaying->b;
                    
                    /* Check if we are at the end */
                    if( curplaying->repeat_count != 1 && curplaying->curpos == curplaying->loopend) {
		       
                        curplaying->curpos = curplaying->looppos;
                        if( curplaying->repeat_count > 1 ) curplaying->repeat_count--;  /* sample doesn't loop forever */

		    } else
		    if( curplaying->repeat_count == 1 && curplaying->curpos == curplaying->playend) {
                         /* Done - delete entry from list */

                            ga->num_playing--;
			    local_num_playing--;

                            if( j != local_num_playing )
                            	ga->playing[j] = ga->playing[local_num_playing];

                            /* Check entry again */
                            curplaying--;
                            j--;
		
		    }

                    /* Next in playlist */
                    curplaying++;
                }
		
		sampAccumL>>=8;
		if(sampAccumL>32767) sampAccumL=32767;
		else
		if(sampAccumL<-32768) sampAccumL=-32768;
		
	        *curposleft=sample16to8(sampAccumL);

                /* Advance pointers */
                curposleft++;

		    
            }
        }

        UNLOCK_BUFFERS(ga);

        /* Write playbuffer to audio device */
        if(CBRflag || fillBackground) { /* Don't send empty buffer unless told to */
	   SEND_BUFFER_TO_DEVICE(playbuffer);
        } else {
	   /* Instead- sleep 1 blocks worth of data -probably better do suspend and get a signal from play() */
	   SLEEP_CALL(blockUsec);
	}
     
     goto playloop;

}


