#include <e32std.h>

#include "sssd.h"
#include "internals.h"
#include "SSSDThreadHandler.h"


_LIT(KSSSDMutexName, "SSSDThreadMutex");

/*
    Start sssdaudio system: open device, setup device
    parameters, initialize tables, start thread.  Returns -1 on error.
    stopAudio does not need to be called if startAudio does not succeed.
 */
int sssd_startAudio( struct sssdaudio *ga ) {

    /* Init variables */
    ga->paused      = 0;
    ga->num_samples = 0;
    ga->num_playing = 0;
    ga->playing_count = 0;
    ga->background_stream = NULL;
    ga->background_stream_bytes = 0;
    ga->lastERR=0;

    /* Allocate arrays */
    ga->playing = 0;
    ga->samples = 0;
    if (ga->max_voices) {
        
        ga->samples=(ga_sample *)malloc(sizeof(struct ga_sample)*ga->max_samples);
        if(ga->samples==NULL) {
           ga->lastERR=ERR_SAMPLES_ALLOC;
           return ERR_SAMPLES_ALLOC;
        }

        ga->playing=(ga_playing *)malloc(sizeof(struct ga_playing)*ga->max_voices);    
        if(ga->playing==NULL) {
           free(ga->samples);
           ga->lastERR=ERR_VOICES_ALLOC;
           return ERR_VOICES_ALLOC;
        }

        if(ga->audioBufferSize==0) {
           ga->audioBufferSize=256*(ga->byps<<1);
           if(ga->freq>22000) ga->audioBufferSize<<=1;
           if(ga->freq>44000) ga->audioBufferSize<<=1;         
        }

        ga->playBuffer=malloc(ga->audioBufferSize);
        if(ga->playBuffer==NULL) {
           free(ga->samples);
           free(ga->playing);
           ga->lastERR=ERR_PLAYBUFFER_ALLOC;
           return ERR_PLAYBUFFER_ALLOC;
        }
        
    }
    
    /* Create Mutex */
    ga->buffer_mutex = NULL;
    RMutex *mutex = new RMutex;
    if (mutex == NULL) {
    	ga->lastERR = ERR_MUTEX_INIT;
    	goto err;
    }
    TInt errcode = mutex->CreateGlobal(KSSSDMutexName);
    if (errcode != KErrNone) {
    	delete mutex;
    	mutex = NULL;
    	ga->lastERR = ERR_MUTEX_INIT;
    	goto err;
    }
    ga->buffer_mutex = (void *)mutex;

    /* Launch mixer background thread */
    CSSSDThreadHandler *handler = NULL;
    ga->platformData = NULL;
    TRAPD(leaveCode, handler = CSSSDThreadHandler::NewL());
    ga->platformData = handler;
    if (leaveCode != KErrNone) {
    	ga->lastERR = ERR_THREAD_START;
    	goto err;
    }
    handler->SSSD_StartAudio(ga);
    
    ga->audio_fd=1;
    ga->lastERR=0;
    return( 0 );
    
err:
	if(mutex != NULL)  { mutex->Close(); delete mutex; ga->buffer_mutex = NULL; }
    if(ga->samples!=NULL) free(ga->samples);
    if(ga->playing!=NULL) free(ga->playing);
    ga->audio_fd = -1;
    return(ga->lastERR);  

}