

#include <stdio.h>
#include <stdlib.h>

#ifndef WIN32
   #include <unistd.h>
#else
   #define usleep(a) _sleep(a/1000);
#endif

#ifndef __ELATE__
#include <sssd.h>     /* libsssd defines */
#else
#include <amiga/sssd.h>
#endif

#include "testregister.h"  /* audio sample as a header */

void printMenu()
{
    printf( "Command Menu:\n"
            "\t? - print menu\n"
            "\t1 - play sample 1\n"
	    "\t2 - play shortened sample\n"
	    "\t3 - play with loop in-out points\n"
            "\tv - play sample 1 changing volume\n"
	    "\tV - play sample 1 changing master volume\n"
            "\tp - play sample 1 changing pan\n"
	    "\tP - play sample 1 changing master pan\n"
            "\tS - Suspend audio\n"
            "\tR - Resume audio\n"
            "\tM - mute audio\n"
            "\tU - un-mute audio\n"
            "\ts - stop all samples\n"
            "\tq - quit demo\n\n" );
}

int main()
{
    int c,i;
    float tmpfloat;

    /* libsssd handle */
    struct sssdaudio *ga;

    /* sample references */
    int fire_id;
            
    /* contrl arrays (can be as big as you like) */
    int parmArray[4];
    float parmDevice[4]; 
    
    printf("Init control struct...(%d)\n",DEFAULT_NUMBUFFERS);
    ga=sssd_initAudio(2,		/* Channels: 1=mono, 2=stereo */
    		 11025,		/* Frequency */
		 8,		/* bits per sample (8|16) */
		 4,		/* Maximum simul voices */
		 2,		/* Maximum samples */
		 0,		/* Hardware audio buffer 0=let system decide */
		 DEFAULT_NUMBUFFERS, /* Number of audio hardware bufffers */
		 0);		/* Flags */
            
    /* open soundcard device */
    printf("Opening device (if Linux, will try ESD first)...\n");
    if( sssd_isError(sssd_startAudio( ga, ESD_DEVICE_NAME ))){
       if( sssd_isError(sssd_startAudio( ga, DEFAULT_DEVICE )) )
       exit(-1);
    }

   /* printf("Latency: %d ms\n",testLatency(ga)); */ /*Buggy */

    /* pre-cache samples ... keeping id's */
    printf( "Registering samples ...\n" );
    fire_id   = sssd_registerSample( ga, (void *)devil_data, devil.chan, devil.freq, devil.type, devil.size, COPY_DEFAULT );

    /* User interface loop */
    printMenu();
    c = 0;
    while( c != 'q' )
    {
        c = getchar();
        switch( c )
        {
            case '1':
                printf( "Playing devil sound once ...\n" );
                sssd_playSample( ga, fire_id, 1, 256, 0 );
                break;

            case '2':
                printf( "Playing shortened devil sound once ...\n" );
                sssd_playSamplePP( ga, fire_id, 0, 1, 256, 0, 5000, 15000, SAMPLE_DEFAULTPOS, SAMPLE_DEFAULTPOS, PRIQ_NONE );
                break;

            case '3':
                printf( "Playing looping in-out devil sound 5 times ...\n" );
                sssd_playSamplePP( ga, fire_id, 0, 5, 256, 0, SAMPLE_DEFAULTPOS, SAMPLE_DEFAULTPOS, 5000, 8000, PRIQ_NONE );
                break;

            case 'l':
                printf( "Playing fire sound in a loop ...\n" ); 
                sssd_playSample( ga, fire_id, 0, 256, 0 );
                break;

            case 'S':
                printf( "Suspending audio ...\n" ); 
                sssd_suspend( ga );
                break;

            case 'R':
                printf( "Resuming suspended audio (if Linux, will switch to OSS)...\n" ); 
                if(sssd_isError(sssd_resume( ga, DEFAULT_DEVICE ))) {
		  puts("Error: Can't resume audio\n");
		  exit(-1);
		}
                break;

            case 'v':
                printf( "Playing devil sound changing volume ... please wait.\n" ); 
                sssd_playSample( ga, fire_id, 0, 256, 0 );
		parmArray[0]=CTRL_S_VOLUME;
		parmArray[2]=CTRL_S_END;
		for (i=0; i<=256; i += 16) {
                 printf ("Volume value set to %i\n",i);
                 parmArray[1]=i;
		 sssd_ctrlSample ( ga, fire_id, parmArray);
                 usleep (500000);
                }
                usleep (1000000);
                sssd_stopSample( ga, fire_id );
                break;

            case 'V':
                printf( "Playing devil sound changing device volume ... please wait.\n" ); 
                sssd_playSample( ga, fire_id, 0, 256, 0 );
		tmpfloat=0;
		parmDevice[0]=CTRL_D_VOLUME;
		parmDevice[2]=CTRL_D_END;
		for (i=0; i<=256; i += 16) {
                 printf ("Device volume value set to %f\n",tmpfloat);
		 parmDevice[1]=tmpfloat;
                 sssd_ctrlDevice ( ga, parmDevice);
                 tmpfloat+=((float)1/16);
		 usleep (500000);
                }
                usleep (1000000);
                sssd_stopSample( ga, fire_id );
                break;

            case 'p':
                printf( "Playing devil sound panning ... please wait.\n" ); 
                sssd_playSample( ga, fire_id, 0, 256, 0 );
		parmArray[0]=CTRL_S_PAN;
		parmArray[2]=CTRL_S_END;
		for (i=-128; i<=128; i += 16) {
                 printf ("Pan value set to %i\n",i);
                 parmArray[1]=i;
                 sssd_ctrlSample ( ga, fire_id, parmArray);
                 usleep (500000);
                }
                usleep (1000000);
                sssd_stopSample( ga, fire_id );
                break;

            case 'P':
                printf( "Playing devil sound changing device pan... please wait.\n" ); 
                sssd_playSample( ga, fire_id, 0, 256, 0 );
		tmpfloat=0;
		parmDevice[0]=CTRL_D_PAN;
		parmDevice[2]=CTRL_D_END;
		for (i=0; i<=256; i += 16) {
                 printf ("Device pan value set to %f\n",tmpfloat);
                 parmDevice[1]=tmpfloat;
		 sssd_ctrlDevice ( ga, parmDevice);
                 tmpfloat+=((float)1/16);
		 usleep (500000);
                }
                usleep (1000000);
                sssd_stopSample( ga, fire_id );
                usleep (1000000);
		parmDevice[0]=CTRL_D_VOLUME;
		parmDevice[2]=CTRL_D_END;
		parmDevice[1]=.5;

		sssd_ctrlDevice ( ga, parmDevice);

                break;

            case 's':
                printf( "Stopping all effects ...\n" );
                sssd_stopAllSamples( ga );
                break;

            case 'M':
                printf( "Mute sound\n" );
                sssd_pauseAudio( ga );
                break;

            case 'U':
                printf( "Un-mute sound\n" );
                sssd_unpauseAudio( ga );
                break;

            case 'h':
            case '?':
                printMenu();
                break;
        }
    }

    /* Shut audio down */
puts("Stop Audio/free device resources");
    sssd_stopAudio( ga ); 

puts("Free memory resources");
    sssd_freeAudio( ga );
    
puts("exit");fflush(stdout);
    return( 0 );
}


/*EOF*/
